class ModelNoneError(Exception):
    def __init__(self, *args, **kwargs):
        pass


class SerializerNoneError(Exception):
    def __init__(self, *args, **kwargs):
        pass


class FilterNoneError(Exception):
    def __init__(self, *args, **kwargs):
        pass
